####Section 2 (of research report): Explanations of the components of  
#### a time series and moving averages


###Packages###
library(fpp3)
library(fpp2)
library(tidyverse)
library(tsibble)
#############


####Trend component in Figure 1###
aus_retail %>%
  filter(`Series ID`=="A3349640L") %>%
  autoplot(Turnover)+
  labs(y = "Turnover (in $)",
       title = "Cafes, restaurants and catering services retail turnover 
       in Victoria, Austrailia")

###Seasonal component###
airmiles
US <- read.table("C:/Users/Simamkele/Downloads/Monthly airmiles.txt")
airmiles <- US$V1
#Airmiles is an R built-in data set however it needed to be transformed into a
#tsibble to use the fpp3 package functions on the data for consistency, so we
#convert it to a text file then time series then a tsibble.
airmiles.ts <- ts(airmiles, frequency = 12,
                  start = c(1996, 1))
airmiles.tsibble <- rename(airmiles.ts %>% as_tsibble,
                           airmiles = value,
                           Date = index)
autoplot(airmiles.tsibble, airmiles) +
  labs(y = "Passenger (miles)",
       title = "Monthly United States airline passenger-miles:
       January 1996 - May 2005.")

###cyclical component###
#The USA Housing data is from FRED Economic data, from the FRED website, with 
#the number of sales of new one-family houses in the United States
US_housing <- read.table("C:/Users/Simamkele/Downloads/USA Housing.txt")
Housing <- US_housing$V1
housing.ts <- ts(Housing, frequency = 12,
                 start = c(1965, 1))
housing.tsibble <- rename(housing.ts %>% as_tsibble,
                          Housing = value,
                          Date = index)
autoplot(housing.tsibble, Housing)+
  labs(y = "Houses(millions))",
       title = "Sales of new one-family houses")

###Moving average of South African exports###
#The global economy data set is also R built-in data.
global_economy %>%
  filter(Country == "South Africa") %>%
  autoplot(Exports) +
  labs(y = "Exports as % of GDP", title = "Total South African exports
       as a % of GDP") #Plot of the data

SA_exports<-global_economy %>%
  filter(Country == "South Africa") %>%
  select(Exports)
SA_exports

#3 order moving average
SA_exports1 <- global_economy %>%
  filter(Country == "South Africa") %>%
  mutate(
    `3-MA` = slider::slide_dbl(Exports, mean,
                               .before = 1, .after = 1, .complete = TRUE)
  )#selecting the South African exports from the global economy data set and 
#and applying the moving average of order 3

SA_exports1 %>%
  autoplot(Exports) +
  geom_line(aes(y = `3-MA`,col=SA_exports1), colour = "#D55E00",lty=2) +
  labs(y = "% of GDP",
       title = "Total South African exports as a percentage of GDP",
       colour="name 1") +
  guides(colour = guide_legend(title = "series"))#plotting the moving average

#5 order moving average
SA_exports <- global_economy %>%
  filter(Country == "South Africa") %>%
  mutate(
    `5-MA` = slider::slide_dbl(Exports, mean,
                               .before = 2, .after = 2, .complete = TRUE)
  )#selecting the South African exports from the global economy data set and 
#and applying the moving average of order 5

SA_exports %>%
  autoplot(Exports) +
  geom_line(aes(y = `5-MA`), colour = "#D55E00",lty=2) +
  labs(y = "% of GDP",
       title = "Total South African exports as a percentage of GDP",
       colour="name 1") +
  guides(colour = guide_legend(title = "series"))#plotting the moving average




