#### Section 3 (of research report): Electricity available for distribution time 
####series data 

####Packages
library(fpp3)
library(forecast)
library(seasonal)

#Read data into R
sa_distributable_electricity <-read.table(
  "C:/Users/Simamkele/Downloads/Electricity available for distribution.txt")

electricity <- sa_distributable_electricity$V1
#Declaring the data as time series data###
electricity.ts <- ts(electricity, frequency = 12,
                     start = c(2000, 1))
#Declaring the data into a tsibble###
electricity.tsibble <- rename(electricity.ts %>% as_tsibble,
                              electricity = value,
                              Date = index)

#Sub-setting the training data set to use through out section 3 of the report.
recent_supply<-electricity.tsibble%>%filter(year(Date)>=2000)#selecting data 
#from Jan 2000 onwards first.
elect_train<-recent_supply%>%
  filter(year(Date)<=2019)#extracting the training set from Jan 2000 to Dec 2019


#Plot of the time series 
autoplot(elect_train, electricity) +
  labs(y = "Total electricity available for distribution (in GW/h)",
       title = "Monthly South African electricity available for distribution
	(in GW/h) from January 2000 to March 2021")


#Monthly boxplots for the time series
Jan<-elect_train$electricity[seq(1, length(elect_train$electricity), 12)]
Feb<-elect_train$electricity[seq(2, length(elect_train$electricity), 12)]
Mar<-elect_train$electricity[seq(3, length(elect_train$electricity), 12)]
Apr<-elect_train$electricity[seq(4, length(elect_train$electricity), 12)]
May<-elect_train$electricity[seq(5, length(elect_train$electricity), 12)]
Jun<-elect_train$electricity[seq(6, length(elect_train$electricity), 12)]
Jul<-elect_train$electricity[seq(7, length(elect_train$electricity), 12)]
Aug<-elect_train$electricity[seq(8, length(elect_train$electricity), 12)]
Sept<-elect_train$electricity[seq(9, length(elect_train$electricity), 12)]
Oct<-elect_train$electricity[seq(10, length(elect_train$electricity), 12)]
Nov<-elect_train$electricity[seq(11, length(elect_train$electricity), 12)]
Dec<-elect_train$electricity[seq(12, length(elect_train$electricity), 12)]
boxplot(Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sept,Oct,Nov,Dec,names=c("Jan","Feb",
                                                                 "Mar","Apr","May","Jun","Jul","Aug","Sept","Oct","Nov","Dec"),
        xlab="Month",ylab="Electricity distributed (Gwh)")



###############################################################################
###Applying the 2x12MA to the electricity data.For the purposes of         
###illustrating the Moving average For Figure  8 in the report.                                    
###############################################################################
electricity_ma<-elect_train %>%
  mutate(
    `12-MA` = slider::slide_dbl(electricity, mean,
                                .before = 5, .after = 6, .complete = TRUE),
    `2x12-MA` = slider::slide_dbl(`12-MA`, mean,
                                  .before = 1, .after = 0, .complete = TRUE)
  )
electricity_ma %>%
  autoplot(electricity, colour = "dark grey") +
  geom_line(aes(y = `2x12-MA`), colour = "#D55E00",lty=2) +
  labs(y = "Electricity(GW/h)",
       title = "Total distributable electricty in Sounth Africa")

#######Implementing Classical decomposition
classical<-elect_train %>%
  model(classical_decomposition=classical_decomposition(electricity, type ="additive"))
components(classical) %>% autoplot()+labs(y = "Electricity available for distribution (GW/h)",title = "Classical additive decomposition of Total electricity 
       available for distribution")
seas1<-components(classical)%>%pull(seasonal)#seasonal component
components(classical)#to view the points that make up the decomposition graphs

randcl<-components(classical)%>%pull(random)#random component
ggAcf(randcl)+
  ggtitle("ACF for classical decomposition random component")#autocorrelation function of residuals alone.
Box.test(randcl,lag=24,fitdf=0,type="Lj")#Ljung-box test.
#Results: X-squared = 168.76, df = 24, p-value < 2.2e-16 Reject H0(Is white noise) at 95% confidence

#######Implementing X11 decomposition
x11_dcmp <- elect_train %>%
  model(x11 = X_13ARIMA_SEATS(electricity ~ x11()))

components(x11_dcmp)%>%autoplot +
  labs(y = "Electricity available for distribution (GW/h)",title ="Decomposition of  Total electicity available for distribution
using X-11.")

randx11<-components(x11_dcmp)%>%pull(irregular)
ggAcf(randx11)+
  ggtitle("ACF for X11 random component")#to obtain the evaluation of the residuals.
Box.test(randx11,lag=24,fitdf=0,type="Lj")#Ljung-box test.
#Results: X-squared = 42.44, df = 24, p-value = 0.01152 Reject H0(Is white noise) at 95% confidence


#######Implementing SEATS decomposition
seats_dcmp <- elect_train %>%
  model(seats = X_13ARIMA_SEATS(electricity~ seats())) 
components(seats_dcmp)%>%autoplot +
  labs(y = "Electricity available for distribution (GW/h)",title ="Decomposition of  Total electicity available for distribution using SEATS")

randseat<-components(seats_dcmp)%>%pull(irregular)#random component
ggAcf(randseat)+
  ggtitle("ACF for SEATS random component")#autocorrelation function of residuals alone.
Box.test(randseat,lag=24,fitdf=0,type="Lj")#Ljung-box test.
#Results:X-squared = 67.208, df = 24, p-value = 5.742e-06 Reject hypothesis of 
#white noise /significantly different from zero.

######Implementing STL Decomposition
dcmp <- elect_train  %>%
  model(stl = STL(electricity~trend(window=7)+
                    season(window="periodic"),robust=TRUE))
components(dcmp)%>% autoplot()+labs(y = "Electricity available for distribution (GW/h)",title = "STL decomposition of Total electricity 
       available for distribution")

#Checking the residuals for white noise.
randstl<-components(dcmp)%>%pull(remainder)#random component
ggAcf(randstl)+
  ggtitle("ACF for STL random component")#autocorrelation function of residuals alone.
Box.test(randstl,lag=24,fitdf=0,type="Lj")
#Results:X-squared = 23.52, df = 24, p-value = 0.4893 Do not reject H0(process represents white noise)

#results in an outlier at lag 20 in the acf plot.
components(dcmp)


######################################
####Forecasting with Decomposition####
######################################
######Damped Holt's method.
fit_dcmp1 <- elect_train %>%
  model(stlf = decomposition_model(
    STL(electricity ~ trend(window = 7)+season(window="periodic"), robust = TRUE),
    ETS(season_adjust ~ error("A") +trend("Ad", phi = 0.9) + season("N"))
  ))

#forecast for January 2020 to April 2021(16 months)
fc_dcmp1<-fit_dcmp1%>%
  forecast(h=16)
accuracy(fc_dcmp1,recent_supply)#obtaining forecasting accuracy, mean,RMSE and
# MAE

######Holt's linear trend
fit_dcmp2 <- elect_train %>%
  model(stlf = decomposition_model(
    STL(electricity ~ trend(window = 7)+season(window="periodic"), robust = TRUE),
    ETS(season_adjust ~ error("A") + trend("A") + season("N"))
  ))

#forecast for January 2020 to April 2021(16 months)
fc_dcmp2<-fit_dcmp2%>%
  forecast(h=16)
accuracy(fc_dcmp2,recent_supply)# obtaining mean,RMSE and MAE 


#####Seasonal Naive 
fit_dcmp3 <- elect_train %>%
  model(stlf = decomposition_model(STL(electricity ~ trend(window = 7)+season(window="periodic"), robust = TRUE),NAIVE(season_adjust))
  )
#forecast for January 2020 to April 2021(16 months)
fc_dcmp3<-fit_dcmp3 %>%
  forecast(h=16)
accuracy(fc_dcmp3,recent_supply)#obtaining mean,RMSE and MAE


#####Mean forecasting method
fit_dcmp4 <- elect_train %>%
  model(stlf = decomposition_model(STL(electricity ~ trend(window = 7)+season(window="periodic"), robust = TRUE),MEAN(season_adjust)
  ))
#forecast for January 2020 to April 2021(16 months)
fc_dcmp4<-fit_dcmp4 %>%
  forecast(h=16)
accuracy(fc_dcmp4,recent_supply)#obtaining mean,RMSE and MAE

